﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Dusza.Reg2011.Boolean {

    /*
     * Osztály egy autó adatainak kezelésére.
     */
    class Auto {

        public string autoId;          // Az autó azonosítója
        public char   jogosultsag;     // Az autó jogosultsága engedélyköteles védett területekhez (`A` - teljeskörű, `B` - fizetős, `C` - nem jogosult)
        public int    szabalytalansag; // Az autóhoz regisztrált szabálytalanságok száma
        public int    egyenleg;        // Az autóhoz tartozó számla egyenlege
        public int    parkolasKezdete; // Parkoló autó esetében a parkolás kezdetének időpontja; egyébként nem értelmezhető

        // Konstruktor az autókhoz, amely az autókat leíró bemeneti fájl sorainak megfelelő stringet vár paraméterül
        public Auto(string s) {
            string[] adatok = s.Split(' ');
            autoId = adatok[0];
            jogosultsag = adatok[1][0];
            szabalytalansag = int.Parse(adatok[2]);
            egyenleg = int.Parse(adatok[3]);
            parkolasKezdete = 0;
        }

        // Az autó adatainak frissítése egy olyan tetszőleges típusú esemény alapján, ami az autóval történt
        public void vegrehajt(Esemeny esemeny) {
            if(autoId != esemeny.autoId) return;
            switch(esemeny.tipus) {
                case Esemeny.EsemenyTipus.Sebessegmeres:
                    athajtas(esemeny);
                    break;
                case Esemeny.EsemenyTipus.Behajtas:
                    behajtas(esemeny);
                    break;
                case Esemeny.EsemenyTipus.Parkolas:
                    parkolas(esemeny);
                    break;
                default: throw new Exception("Ismeretlen eseménytípus (" + esemeny.tipus + ").");
            }
        }

        // Az autó szabálytalanságait és egyenlegét leíró szöveges információt visszaadó függvény
        public string kiiratas() {
            return "Szabálytalanságok száma: " + szabalytalansag.ToString()  + Environment.NewLine +
                   "Aktuális egyenleg: "       + egyenleg.ToString() + " Ft" + Environment.NewLine;
        }

        // Az autó adatbázisban eltárolt adatait a mentéshez szükséges formátumban visszaadó függvény
        public string exportalas() {
            return autoId + " " + jogosultsag + " " + szabalytalansag + " " + egyenleg;
        }

        // Az autó adatainak frissítése egy olyan áthajtási esemény alapján, ami az autóval történt
        private void athajtas(Esemeny esemeny) {
            if (esemeny.gyorshajtas) szabalytalankodas();
        }

        // Az autó adatainak frissítése egy olyan behajtási esemény alapján, ami az autóval történt
        private void behajtas(Esemeny esemeny) {
            if(!esemeny.irany) return;
            switch(esemeny.teruletTipus) {
                case 1:
                    szabalytalankodas();
                    break;
                case 2:
                    egyenleg += 300;
                    break;
                case 3:
                    switch(jogosultsag) {
                        case 'A': break;
                        case 'B':
                            egyenleg += 300;
                            break;
                        case 'C':
                            szabalytalankodas();
                            break;
                        default: throw new Exception("Ismeretlen jogosultság (" + jogosultsag + ").");
                    }
                    break;
                default: throw new Exception("Ismeretlen területtípus (" + esemeny.teruletTipus + ").");
            }
        }

        // Az autó adatainak frissítése egy olyan parkolási esemény alapján, ami az autóval történt
        private void parkolas(Esemeny esemeny) {
            if(esemeny.irany) {
                parkolasKezdete = esemeny.idopont;
            } else {
                if(szabalytalansag < 2) return;
                int parkolasiIdo = esemeny.idopont - parkolasKezdete;
                // A másodpercekben vett parkolási időt át kell konvertálni a megkezdett órák számára
                parkolasiIdo = Convert.ToInt32(Math.Ceiling(parkolasiIdo / 3600.0));
                egyenleg += parkolasiIdo * 200;
            }
        }

        // Az autó adatainak frissítése egy szabály megszegésének esetén
        private void szabalytalankodas() {
            szabalytalansag++;
            if(szabalytalansag == 5) egyenleg += 20000;
        }
    }
}
