﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Dusza.Reg2011.Boolean {

    /*
     * Osztály egy esemény adatainak kezelésére.
     */
    class Esemeny {

        // Lehetséges eseménytípusok
        public enum EsemenyTipus {
            Sebessegmeres,
            Behajtas,
            Parkolas
        }

        public EsemenyTipus tipus;           // Az esemény típusa
        public string       autoId;          // Az eseményhez tartozó autó azonosítója
        public int          idopont;         // Az első kapu által rögzített időpont
        public bool         irany;           // A haladási irány, `true` behaladásnál (ha először a kisebb sorszámú leolvasóval találkozik a kocsi)
        public bool         gyorshajtas;     // `True`, ha az áthaladó autó sebessége átlépi a sebességhatárt; csak `Sebessegmeres` típusú eseményeknél értelmezhető
        public int          teruletTipus;    // A védett terület típusa (1 - tiltott, 2 - fizetős, 3 - engedélyköteles); csak `Behajtas` típusú eseményeknél értelmezhető

        // Konstruktor az eseményekhez, amely az eseményeket tartalmazó bemeneti fájl sorainak megfelelő stringet vár paraméterül
        public Esemeny(string s) {
            string[] adatok = s.Split(' ');
            string olvasoId = adatok[1];
            autoId = adatok[0];
            idopont = Esemeny.masodperc(adatok[2]);

            // Az olvasó azonosítójának első számjegye határozza meg az olvasó által rögzített esemény típusát.
            int olvasoTipus = (int)char.GetNumericValue(olvasoId[0]);
            switch(olvasoTipus) {
                case 1:
                case 2:
                    tipus = EsemenyTipus.Sebessegmeres;
                    break;
                case 3:
                case 4:
                    tipus = EsemenyTipus.Behajtas;
                    break;
                case 5:
                case 6:
                    tipus = EsemenyTipus.Parkolas;
                    break;
                default: throw new Exception("Ismeretlen eseménytípus (" + olvasoTipus + ").");
            }

            // Az olvasópárok közül a kisebb sorszámú azonosítójának első számjegye mindig páratlan
            irany = olvasoTipus % 2 == 1;

            if(tipus == EsemenyTipus.Sebessegmeres) {
                int idotartam = Esemeny.masodperc(adatok[3]) - idopont;
                // A szenzorok között 100 méter a távolság, m/s és km/h között pedig 3.6 a váltószám
                double sebesseg = 360.0 / idotartam; 
                gyorshajtas = int.Parse(adatok[1].Substring(1, 2)) < (int)Math.Floor(sebesseg);
            } else if(tipus == EsemenyTipus.Behajtas) {
                teruletTipus = olvasoId[1] - '0';   // Mivel `teruletTipus` int típusú, `olvasoId[1]` pedig karakterként tárolt számjegy, a helyes konverzióhoz ki kell vonni a `0` karakter kódját
            }
        }

        // A 'HH:mm:ss' formátumú időpontok nap eleje óta eltelt másodpercek formátumra való átalakítása
        private static int masodperc(string s) {
            string[] adatok = s.Split(':');
            return int.Parse(adatok[0]) * 3600 + int.Parse(adatok[1]) * 60 + int.Parse(adatok[2]);
        }
    }
}
