﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace Dusza.Reg2011.Boolean  {
    public partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();
        }

        Dictionary<string, Auto> autok;     // A beolvasott autók adatait eltároljuk a memóriában, hogy megjeleníthessük az adatokat kérés esetén

        /*
         * Autók adatainak beolvasása egy fájlból.
         */
        public void importAutok(string filename) {
            string[] lines;
            autok = new Dictionary<string, Auto>();
            try {
                lines = File.ReadAllLines(filename);
                for(int i = 0; i < lines.Length; i++) {
                    Auto auto = new Auto(lines[i]);
                    autok.Add(auto.autoId, auto);
                }
            } catch (Exception e) {
                throw new Exception("Hiba az autók beolvasásánál: " + e.Message);
            }
        }

        /*
         * Események adatainak beolvasása egy fájlból, majd az autók adatainak frissítése az események alapján.
         */
        private void vegrehajtEsemenyek(string fileName) {
            string[] adatok;
            try {
                adatok = File.ReadAllLines(fileName);
                List<Esemeny> esemenyek = new List<Esemeny>();
                for(int i = 0; i < adatok.Length; i++) {
                    if (adatok[i] != "") esemenyek.Add(new Esemeny(adatok[i]));
                }
                // Az eseményeket rendezzük időpont alapján, mivel több esetben is fontos az időrendiség (pl. 2 szabálytalanság után már az aznapi parkolás is pénzbe kerül),
                // azonban a bemenet sorai nem biztos hogy időrendben következnek.
                esemenyek.Sort(delegate(Esemeny e1, Esemeny e2) { return e1.idopont.CompareTo(e2.idopont); });
                foreach(var esemeny in esemenyek) {
                    if(!autok.ContainsKey(esemeny.autoId)) throw new Exception("Ismeretlen autó azonosító.");
                    autok[esemeny.autoId].vegrehajt(esemeny);
                }
            } catch(Exception e) {
                throw new Exception("Hiba az események beolvasásánál: " + e.Message);
            }
        }

        /*
         * Autók adatainak kiírása egy fájlba.
         */
        private void exportAutok(string fileName) {
            StreamWriter sw = new StreamWriter(fileName);
            string[] s = new string[autok.Count];
            int x = 0;
            foreach(Auto a in autok.Values) s[x++] = a.exportalas();
            for(int i = 0; i < s.Length; i++) {
                if(i != s.Length - 1) {
                    sw.WriteLine(s[i]);
                } else sw.Write(s[i]);
            }
            sw.Close();
        }

        /*
         * A `Megjelenít` gomb eseménykezelője. Az elvárt működés szerint ki kell iratni az adott azonosítójú autó adatait.
         */
        private void button1_Click(object sender, EventArgs e) {
            if(bevitel.Text != "") {
                pictureBox1.Visible = false;
                if(!autok.ContainsKey(bevitel.Text)) {
                    kimenet.Text = "Nem található ilyen rendszámú autó!";
                    return;
                }
                kimenet.Text = autok[bevitel.Text].kiiratas();
            } else {
                kimenet.Text = "";
                pictureBox1.Visible = true;
            }
        }

        /*
         * Az alkalmazás indulásakor még az ablak megjelenítése előtt betöltjük az autók és események adatait, illetve frissítjük az autókat leíró fájlt.
         */
        private void MainForm_Load(object sender, EventArgs e) {
            try {
                importAutok("chip.txt");
                vegrehajtEsemenyek("olvas.txt");
                exportAutok("chip.txt");
            } catch (Exception ex) {
                MessageBox.Show("Hiba történt a fájlok beolvasása során: " + ex.Message);
                Close();
                return;
            }
        }
    }
}
